<?php

/** Main Controller
    CS 174 Fall 2010
    
    @package hw4
    @author SJSU Student
    */

class main
{
    /** Entry function.
        Gets fortunes.
        */
        
    function entry()
    {
        // ------[Get the GET Variables]------ //
        
        // url_grader_is_grading_from/index.php?num=XXX&genre=XXX
        //     &rating=XXX&source=XXX&format=XXX
        
        $num    = $_GET["num"];
        $genre  = strtolower($_GET["genre"]);
        $rating = strtolower($_GET["rating"]);
        $source = strtolower($_GET["source"]);
        $format = strtolower($_GET["format"]);
        
        // ------[Sanitize/fix GET Variables]------ //
        
        if (empty($num) || !is_numeric($num) || $num < 1) $num = 5;
        if (empty($genre)) $genre = "Quotations";
        if (empty($rating)) $rating = "G";
        if (empty($source)) $source = "all"; // default to "all"
        if ($source != "internal" && $source != "external")
            $source = "all"; // bad source value, goto default!
        if ($format != "json") $format = "xml";
        
        // ------[Grab Fortunes]------ //
        
        $fortunes = array();
        global $internal_source;
        $internal_xml = file_get_contents($internal_source);
        
        for ($i = 0; $i < $num; $i++)
        {
            // pick a fortune source:
            if (($source == "external") ||
                ($source == "all" && rand(0, 1) == 1))
                $fortunes[] = main::getExternal($rating);
            else $fortunes[] = main::getInternal($internal_xml, $rating);
        }
        
        // ------[If Necessary, Change Format]------ //
        
        if ($format == "json") {
            $comma ="";
            $out = "[";
            for ($i = 0; $i < $num; $i++) {
                $out .= $comma.main::toJSON($fortunes[$i]);
                $comma= ",";
            }
            $fortunes = $out."]";
        }
        
        // ------[Display Fortunes]------ //
        
        $data["page_title"] = "CS 174: Fortune Favors Fools";
        $data["fortunes"] = $fortunes;
        
        $window = $format."_view";
        require_once("./views/".$window.".php");
        $view = new $window();
        $view->render($data);
    }
    
    // ------[Helper Functions]------//
    
    /** Grabs an external fortune.
        @param rating the desired fortune rating (G, PG, etc)
        @return an externally-retrieved fortune
        */
    
    private function getExternal($rating)
    {
        // get fortune:
        global $external_source;
        $moe = curl_init($external_source);
        curl_setopt($moe, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($moe);
        curl_close($moe);
        
        // move to xml:
        $start  = stripos($result, "<p>");
        $end    = stripos($result, "</p>");
        $result = substr($result, $start, $end-$start);
        $result = str_ireplace("<p>", "<item><text>", $result);
        $result = str_ireplace("<br>", "", $result);
        if ($result[strlen($result)-1] == "\n");
            $result = substr($result, 0, strlen($result)-1); // chop!
        
        $result .= "</text><rating>G</rating>"
            ."<websource>".$external_source."</websource></item>";
        
        if ($rating == "r" || $rating == "x")
            $result = main::inbedify($result);
        
        // return fortune:
        return $result;
    }
    
    /** Grabs an internal fortune.
        @param xml from which to draw a fortune
        @param rating desired rating (G, PG, etc)
        @return an internally-retrieved fortune
        */
    
    private function getInternal($internal_xml, $rating)
    {
        // select a random fortune from internal source:
        global $internal_max_fortune;
        $fort_num = rand(0, $internal_max_fortune);
        for ($i = -1, $start = -1; $i < $fort_num; $i++)
            $start = stripos($internal_xml, "<item", $start+1);
        $end = stripos($internal_xml, "</item>", $start) + 7;
        $result = substr($internal_xml, $start, $end-$start);
        
        // deal with rating pickiness
        if ($rating == "r" || $rating == "x")
            $result = main::inbedify($result);
        // 'else' doesn't matter -- all internal fortunes are tame :-/
        
        // return fortune:
        return $result;
    }
    
    /** Converts an XML fortune to JSON.
        @param xml the xml to convert
        @return a JSON equivalent of the xml sent
        */
    
    private function toJSON($xml)
    {
        // start json (fortune's text is assumed to exist):
        $json = "{ \"item\": {\n\t\"text\": \"";
        
        // add fortune text to json:
        $start_text = stripos($xml, "<text");
        $start_text = stripos($xml, ">", $start_text+1)+1; // id attrib.
        $end_text   = stripos($xml, "</text>");
        $json .= substr($xml, $start_text, $end_text-$start_text)."\"";
        
        // add the optional tags to json if needed:
        $tags = array("genre", "rating", "author", "websource");
        
        for ($i = 0; $i < count($tags); $i++)
        {
            $start = stripos($xml, $tags[$i]) + strlen($tags[$i])+1;
            if ($start > strlen($tags[$i])+2)
            {
                $end = stripos($xml, "</".$tags[$i].">");
                $json .= ",\n\t\"".$tags[$i]."\": \""
                    .substr($xml, $start, $end-$start)."\"";
            }
        }
        
        // return json:
        return $json."\n\t}\n}";
    }
    
    /** Adds "in bed" to a fortune. Silly.
        @param fortune the fortune to modify
        @return the fortune with "in bed" added to the text
        */

    private function inbedify($fortune)
    {
        $endtext = stripos($fortune, "</text>");
        $x = strlen($fortune)-$endtext;
        $lastper = strrpos($fortune, ".", $endtext-strlen($fortune));
        
        $lastexc = strrpos($fortune, "!", $endtext-strlen($fortune));
        if ($lastper > $lastexc) // assumes some punctuation
            $fortune = substr_replace($fortune, " in bed", $lastper, 0);
        else $fortune = substr_replace($fortune, " in bed", $lastexc, 0);
        
        return $fortune;
    }
}

?>
